import { sequelize } from '@/modules/sequelize.ts';
import { DataTypes, Model } from 'sequelize';
import dayjs from 'dayjs';

export interface VipData {
  [key: string]: any;
  payInfo?: any;
}

export type Vip = Partial<InstanceType<typeof VipModel>>;

export enum VipLevel {
  FREE = 'free',
  LOVE = 'love',
  VIP = 'vip',
}

export enum VipCategory {
  CENTER = 'center',
  AI_CHAT = 'ai-chat',
}

export class VipModel extends Model {
  declare id: string;
  declare userId: string;
  declare title: string;
  declare description: string;

  declare level: VipLevel;
  declare category: VipCategory;
  declare startDate: Date;
  declare endDate: Date;
  declare data: VipData;

  declare createdAt: Date;
  declare updatedAt: Date;
  addVip(opts: { level?: string; month?: number }) {
    const { level, month = 1 } = opts;
    if (level === 'free') return;
    const isSameLevel = this.level === level;
    let endDate = this.endDate;
    // 检测是否过期
    const isExpired = dayjs(endDate).isBefore(dayjs());
    if (isExpired) {
      endDate = new Date();
    }
    if (!isSameLevel) {
      endDate = new Date(); // 直接硬覆盖，love 和 vip 不能叠加
    }
    const newEndDate = dayjs(endDate).add(month, 'month').toDate();
    this.endDate = newEndDate;
    this.level = level as VipLevel;
    this.save({
      fields: ['endDate', 'level'],
    });
  }
}

VipModel.init(
  {
    id: {
      type: DataTypes.UUID,
      primaryKey: true,
      defaultValue: DataTypes.UUIDV4,
    },
    userId: {
      type: DataTypes.UUID,
      allowNull: false,
    },
    title: {
      type: DataTypes.TEXT,
      allowNull: false,
      defaultValue: '',
    },
    description: {
      type: DataTypes.TEXT,
      allowNull: false,
      defaultValue: '',
    },

    level: {
      type: DataTypes.STRING,
      defaultValue: VipLevel.FREE,
    },
    category: {
      type: DataTypes.STRING,
      allowNull: false,
    },
    startDate: {
      type: DataTypes.DATE,
      defaultValue: DataTypes.NOW,
    },
    endDate: {
      type: DataTypes.DATE,
      allowNull: true,
    },
    data: {
      type: DataTypes.JSONB,
      defaultValue: {},
    },
  },
  {
    sequelize,
    tableName: 'kv_vip',
    paranoid: true,
  },
);

VipModel.sync({ alter: true, logging: false }).catch((e) => {
  console.error('VipModel sync', e);
});
