import { app, router } from '@/app.ts';
import { AlipayNotify, TradeStatus } from '../routes/pay/alipay/notify.ts';
import { TradeModel } from '@/routes/trade/model.ts';
import { log } from '@/logger/index.ts';
import { pick } from 'lodash-es';
import { parseBody } from './get-body.ts';
import { verifySignature } from '@/routes/pay/wechatpay/init.ts';
import { wxStateToAlipayState } from '@/routes/pay/wechatpay/notify.ts';

router.get('/api/s1/pay/notify/alipay', async (req, res) => {
  res.end('this api is available for alipay callback');
});

export const getPickData = (body: any, type: 'alipay' | 'wechatpay') => {
  let pickNeedBody = {};
  if (type === 'alipay') {
    // 支付宝
    // trade_status 交易状态 TRADE_SUCCESS, TRADE_CLOSED, TRADE_FINISHED, TRADE_FAILED
    pickNeedBody = pick(body, ['out_trade_no', 'trade_status', 'money', 'subject']);
  }
  if (type === 'wechatpay') {
    pickNeedBody = {
      out_trade_no: body.out_trade_no,
      trade_status: body.trade_state, // 交易状态 SUCCESS, REFUND, NOTPAY, CLOSED, REVOKED, USERPAYING, PAYERROR
      money: body.amount?.total, // 金额
      subject: body.description || '', // 商品描述
    };
  }
  return { pickNeedBody, body };
};
router.post('/api/s1/pay/notify/alipay', async (req, res) => {
  try {
    // 获取支付宝POST过来反馈信息
    const body = (await parseBody(req, res)) as AlipayNotify;
    const { pickNeedBody, body: bodyData } = getPickData(body, 'alipay');

    log.info('pay notify alipay post', { body: pickNeedBody });
    const out_trade_no = body.out_trade_no;
    const status = body.trade_status as TradeStatus;
    const trade = await TradeModel.findOne({
      where: {
        out_trade_no,
      },
    });
    if (!trade) {
      log.error('trade not found', { out_trade_no, ...pickNeedBody });
      res.end('success');
      return;
    }
    trade.update({
      status,
      data: {
        ...trade.data,
        list: [...trade.data.list, body],
      },
    });
    if (status === TradeStatus.TRADE_SUCCESS) {
      app.call(
        {
          path: 'trade',
          key: 'paySuccess',
        },
        {
          trade,
        },
      );
    }
  } catch (e) {
    log.error('pay notify alipay error', {
      message: e.message,
      stack: e.stack,
    });
  }
  res.end('success');
});

router.post('/api/s1/pay/notify/wx', async (req, res) => {
  let wxResult = {};
  const body = await parseBody(req, res);
  try {
    const { result, decryptBody } = await verifySignature(req, body);
    res.writeHead(200, { 'Content-Type': 'text/plain' });
    res.end(result ? 'success' : 'verify failed');
    if (!result) {
      log.error('pay notify wx verify error', {
        id: body?.id,
        create_time: body?.create_time,
        resource_type: body?.resource_type,
        event_type: body?.event_type,
        summary: body?.summary,
      });
      return;
    }
    wxResult = decryptBody;
  } catch (e) {
    log.error('pay notify wx error', {
      message: e.message,
      body,
    });
    return;
  }
  try {
    const { pickNeedBody, body: bodyData } = getPickData(wxResult, 'wechatpay');
    log.info('pay notify wx post', { body: pickNeedBody });
    const out_trade_no = bodyData.out_trade_no;
    const status = bodyData.trade_state;
    const trade = await TradeModel.findOne({
      where: {
        out_trade_no,
      },
    });
    if (!trade) {
      log.error('trade not found', { out_trade_no, ...pickNeedBody });
      res.end('success');
      return;
    }
    trade.update({
      status: wxStateToAlipayState[status],
      data: {
        ...trade.data,
        list: [...trade.data.list, bodyData],
      },
    });
    if (status === 'SUCCESS') {
      app.call(
        {
          path: 'trade',
          key: 'paySuccess',
        },
        {
          trade,
        },
      );
    }
  } catch (e) {
    log.error('pay notify wx notify trade error', {
      message: e.message,
      stack: e.stack,
    });
  }
});
